# Generated by Django 3.2.18 on 2023-10-25 02:06

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="ExpiredNodesRecord",
            fields=[
                ("id", models.BigAutoField(primary_key=True, serialize=False, verbose_name="ID")),
                ("nodes", models.TextField(verbose_name="到期节点信息")),
            ],
            options={
                "verbose_name": "到期节点数据记录 ExpiredNodesRecord",
                "verbose_name_plural": "到期节点数据记录 ExpiredNodesRecord",
            },
        ),
        migrations.CreateModel(
            name="NodeTimerEventConfig",
            fields=[
                ("root_pipeline_id", models.CharField(max_length=64, verbose_name="root pipeline id")),
                (
                    "node_id",
                    models.CharField(max_length=64, primary_key=True, serialize=False, verbose_name="task node id"),
                ),
                ("events", models.TextField(default="[]", verbose_name="timer events")),
            ],
            options={
                "verbose_name": "节点时间事件配置 NodeTimerEventConfig",
                "verbose_name_plural": "节点时间事件配置 NodeTimerEventConfig",
                "index_together": {("root_pipeline_id", "node_id")},
            },
        ),
    ]
