# Generated by Django 3.2.23 on 2023-12-29 09:11

import pipeline.contrib.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("rollback", "0002_auto_20231020_1234"),
    ]

    operations = [
        migrations.AddField(
            model_name="rollbackplan",
            name="options",
            field=pipeline.contrib.fields.SerializerField(default={}, verbose_name="rollback options"),
        ),
    ]
